/*
    Minivader emulator for Sega Saturn
    by Charles MacDonald
    WWW: http://cgfm2.emuviews.com

    Based on the MAME driver by Takahiro Nogi.
    Uses the MAME Z80 CPU emulator as well. (www.mame.net)

    Uses the SH-COFF toolchain from KPIT Infosystems. (www.kpit.com)
    Load and run at address $06004000.

    Controls:

    Left/Right - Move player from side to side
    B          - Add credits
    C          - Fire
    Start      - Return to cheat cartridge menu

    The program does not run at full speed. The SH-2 doesn't quite have
    enough speed to emulate the Z80, though I've tried to add as many
    speed hacks as possible to compensate.

    You need to convert the binary data from the Minivader ROM image into a
    header file, which is included below as "rom.h".

    I've included a dummy program which does nothing but fill the display
    with vertical stripes.
*/

#include <stdio.h>
#include <string.h>
#include "shared.h"
#include "z80.h"
#include "rom.h"

/* From render.s */
extern void render_display(void);
uint8 *ram = (uint8 *)0x00210000;

unsigned int cpu_readport16(unsigned int port)
{
    return 0xFF;
}

void cpu_writeport16(unsigned int port, unsigned int data)
{
}

void _main()
{
    uint16 joy_old, joy_new, joy_delta;

    vdp_init();
    conio_init();
    pad_init();

    /* Screen on (320x240) */
    TVMD = 0x8010;

    /* Set up memory map */
    memset(ram, 0xFF, 0x10000);
    memset(ram+0xA000, 0x00, 0x2000);
    memcpy(ram, rom, 0x2000);

    ram[0x00CC] = 0x00;
    ram[0x00CD] = 0x00;
    ram[0x00CE] = 0x00;

    ram[0x013E] = 0x00;
    ram[0x013F] = 0x00;
    ram[0x0140] = 0x00;

    ram[0x0168] = 0x00;
    ram[0x0169] = 0x00;
    ram[0x016A] = 0x00;

    ram[0x0176] = 0x00;
    ram[0x0177] = 0x00;
    ram[0x0178] = 0x00;

    z80_reset(0);

    for(;;)
    {
        uint8 temp;

        z80_execute(228*262);
        z80_set_irq_line(0, ASSERT_LINE);
        z80_set_irq_line(0, CLEAR_LINE);

        render_display();

        /* Poll joypad */
        joy_old = joy_new;
        joy_new = pad_read(0);
        joy_delta = (joy_new ^ joy_old) & joy_new;

        /* Start - Quit program */
        if(joy_new & PAD_START)
            break;

        /* Update emulator buttons */
        temp = 0xFF;
        if(joy_new & PAD_LEFT)      temp &= ~0x01;
        if(joy_new & PAD_RIGHT)     temp &= ~0x02;
        if(joy_new & PAD_C)         temp &= ~0x04;
        if(joy_new & PAD_B)         temp &= ~0x08;
        ram[0xE008] = temp;
    }
}

